#define STICKY_GNOME_C

#include "main.h"

#ifdef USE_GNOME

static GnomeClient *client = NULL;

static const char *changed_dir = NULL;

static int sticky_gnome_save_session(GnomeClient *client, int phase,
                                     GnomeRestartStyle save_style,
                                     int shutdown,
                                     GnomeInteractStyle interact_style,
                                     int fast, gpointer client_data)
{
 const char *argv[10];
 char *session_id = gnome_client_get_id(client);
 int count = 0;

 IGNORE_PARAMETER(phase && save_style && shutdown && interact_style && fast);
 
 argv[count++] = (char *) client_data;

 if (changed_dir)
 {
  argv[count++] = "--sticky-dir";
  argv[count++] = changed_dir;
 }

 if (sticky_win_x_offset)
 {
  char buffer[64];

  sprintf(buffer, "%d", sticky_win_x_offset);
  
  argv[count++] = "--window-x-offset";
  argv[count++] = g_strdup(buffer);
 }
 
 if (sticky_win_y_offset)
 {
  char buffer[64];

  sprintf(buffer, "%d", sticky_win_y_offset);
  
  argv[count++] = "--window-y-offset";
  argv[count++] = g_strdup(buffer);
 }
 
 gnome_client_set_clone_command(client, count, (char **) argv);
 gnome_client_set_restart_command(client, count, (char **) argv);

 argv[count++] = "--discard-session";
 argv[count++] = session_id;
 
 gnome_client_set_discard_command(client, count, (char **) argv);

 {
  char *session_name = g_copy_strings("session-", session_id, NULL);
  /* not sure if session_id's can be different lengths...
   * so put the .tmp on the front of the filename */
  char *session_name_tmp = g_copy_strings("tmp_session-", session_id, NULL);
  file_io io;

  if (file_write_open(session_name_tmp, STICKY_SESSION_FILE_VERSION, &io))
  {
   /* save any data we want to... */
   file_write_close(&io);
  }

  rename(session_name_tmp, session_name);
  
  free(session_name);
  free(session_name_tmp);
 }

 return (TRUE);
}

static void sticky_gnome_discard_session(const char *session_id)
{
 char *session_name = g_copy_strings("session-", session_id, NULL);
 
 remove(session_name);
 
 free(session_name);
}

static void sticky_gnome_load_session(GnomeClient *client, int restarted,
                                      gpointer ignore)
{
 char *session_id = gnome_client_get_previous_id(client);
 
 IGNORE_PARAMETER(ignore);
 
 if (restarted && session_id)
 {
  char *session_name = g_copy_strings("session-", session_id, NULL);
  file_io io;

  if (file_write_open(session_name, 1, &io))
  {
   file_write_close(&io);
  }
  
  free(session_name);
 }

 if (session_id)
   sticky_gnome_discard_session(session_id);
}

static error_t sticky_gnome_parse_arg(int key, char *arg,
                                      struct argp_state *state)
{
 IGNORE_PARAMETER(state);
 
 switch (key)
 {
  case STICKY_DIR_KEY:
  {
   if (changed_dir)
   {
    fprintf(stderr, " You can only specify one base dir.\n");
    exit (EXIT_FAILURE);
   }
   
   if (chdir(arg))
   {
    mkdir(arg, 0700);
    if (chdir(arg))
    {
     fprintf(stderr, " Could not cd to (or create) specified directory.\n");
     exit (EXIT_FAILURE);
    }
   }
   
   changed_dir = arg;
  }
  break;

  case WIN_OFFSET_BOTH_KEY:
    sticky_win_x_offset = sticky_win_y_offset = atoi(arg);
  break;
  case WIN_OFFSET_X_KEY:
    sticky_win_x_offset = atoi(arg);
  break;
  case WIN_OFFSET_Y_KEY:
    sticky_win_y_offset = atoi(arg);
  break;
  
  case DISCARD_KEY:
    sticky_gnome_discard_session(arg);
    exit (EXIT_SUCCESS);
    break; /* never got to */
    
  default:
    return (ARGP_ERR_UNKNOWN);
 }

 return (0);
}

void sticky_gnome_main(int argc, char *argv[])
{
 static struct argp_option arguments[] =
 {
  {"discard-session", DISCARD_KEY, N_("ID"), 0, N_("Discard session"), 1},
  {"window-offset", WIN_OFFSET_KEY, N_("OFFSET"), 0, N_("Offset for window startup"), 0},
  {"sticky-dir", STICKY_DIR_KEY, N_("DIR"), 0,
   N_("Change the base directory"), 0},
  { NULL, 0, NULL, 0, NULL, 0 }
 };
 static struct argp parser =
 {
  arguments, /* Options.  */
  sticky_gnome_parse_arg, /* The parser function.  */
  NULL, /* Some docs.  */
  NULL, /* Some more docs.  */
  NULL, /* Child arguments -- gnome_init fills this in for us.  */
  NULL, /* Help filter.  */
  NULL /* Translation domain; for the app it can always be NULL.  */
 };
 
 argp_program_version = "(0.8) from the sticky-notes package, by James Antill";
 
 client = gnome_client_new_default();
 
 gtk_signal_connect(GTK_OBJECT(client), "save_yourself",
                    GTK_SIGNAL_FUNC(sticky_gnome_save_session),
                    (gpointer) argv[0]);
 gtk_signal_connect(GTK_OBJECT(client), "connect",
                    GTK_SIGNAL_FUNC(sticky_gnome_load_session), NULL);

 /* FIXME: this generates a warning... it's gnome's bad typing fault :( */
 gnome_init("sticky_note_server", &parser, argc, argv, 0, NULL);
 
 if (!changed_dir)
 {
  if (chdir(getenv("HOME")))
  {
   fprintf(stderr, " Could not cd to home directory.\n");
   exit (EXIT_FAILURE);
  }
  
  if (chdir(".sticky"))
  {
   mkdir(".sticky", 0700);
   if (chdir(".sticky"))
   {
    fprintf(stderr, " Could not cd to (or create) .sticky directory in "
            "your home directory.\n");
    exit (EXIT_FAILURE);
   }
  }
 }
}

#endif
